import 'dart:async';

import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../model/java_script_channel.dart';

class ListenableViewModel extends ChangeNotifier {
  ListenableViewModel.loadRequest({required Uri requestedUrl}) {
    _controller = WebViewController();
    _controller.loadRequest(requestedUrl);
  }

  ListenableViewModel.loadHtmlString({required String html, String? baseUrl}) {
    _controller = WebViewController();
    _controller.loadHtmlString(html, baseUrl: baseUrl);
  }

  ListenableViewModel.loadFile({required String absoluteFilePath}) {
    _controller = WebViewController();
    _controller.loadFile(absoluteFilePath);
  }

  /// 外部访问WebViewController的接口
  late WebViewController _controller;
  WebViewController get controller => _controller;

  /// 外部访问JavaScriptChannel的接口
  final List<JavaScriptChannel> _javaScriptChannels = [];
  List<JavaScriptChannel>? get javaScriptChannels => _javaScriptChannels;
  Future<bool> setJavaScriptChannels(JavaScriptChannel channel) async {
    if (_javaScriptChannels
        .where((element) => element.name == channel.name)
        .isNotEmpty) {
      return Future.value(false);
    }
    _javaScriptChannels.add(channel);
    controller.addJavaScriptChannel(channel.name,
        onMessageReceived: channel.channelCallback.run);
    controller.reload();
    return Future.value(true);
  }

  Future<bool> removeJavaScriptChannel(String name) async {
    if (_javaScriptChannels.where((element) => element.name == name).isEmpty) {
      return Future.value(false);
    }
    _javaScriptChannels.removeWhere((element) => element.name == name);
    controller.removeJavaScriptChannel(name);
    controller.reload();
    return Future.value(true);
  }

  Future<bool> showJavaScriptLog(
      void Function(JavaScriptConsoleMessage message) onConsoleMessage) async {
    controller.setOnConsoleMessage(onConsoleMessage);
    return Future.value(true);
  }
}
